import argparse
import hashlib
import json
import os
import plistlib

PROFILES_PLIST_PATH = 'profiles/profiles.plist'
PROFILES_DIR_PATH = 'profiles/'
PROFILES_OUTPUT_PATH = 'profiles/JsonFormat/'

PARAMETERS_PLIST_PATH = 'profiles/parameters.plist'

DOWNLOADS_PATH = 'downloads-dev.plist'

SECRET_HASH_KEY = 'ngs'

def xcode_plist_adapt(jsonString):
    # xcode saves ' as &apos, replace before calculating hash
    jsonString.replace("'", "&apos")

def calculateHash(dataJson):
    hsh = hashlib.sha1()
    hsh.update(dataJson.encode('utf-8'))
    hsh.update(SECRET_HASH_KEY.encode('utf-8'))
    return hsh.hexdigest()

def convertDataToJson(data):
    dataJson = json.dumps(data, sort_keys=True, separators=(',', ':'), ensure_ascii=False)
    xcode_plist_adapt(dataJson)
    return dataJson

def saveToFile(dataString, outputPath, filename):
    filepath = os.path.join(outputPath, filename)
    with open(filepath, 'w') as output:
        output.write(dataString)

def readPlist(path):
    with open(path, 'rb') as fp:
        plist = plistlib.load(fp)
    return plist

def generateProfilesJson(outputPath):
    print('-- Converting profiles.plist to json --')
    profilesHashes = {}
    profilesDir = os.path.join(currentdir, PROFILES_DIR_PATH)
    for file in os.listdir(profilesDir):
        profilePath = os.path.join(profilesDir, file)
        filename = os.path.splitext(file)[0]
        ext = os.path.splitext(file)[-1]
        print(filename)
        print(ext)
                
        if not os.path.isfile(profilePath) or ext != '.plist':
            print('exiting')
            continue
        print('Reading ' + file)
        profilePlist = readPlist(profilePath)
        profileJson = convertDataToJson(profilePlist)
        
        saveToFile(profileJson, outputPath, filename + '.json')
        itemHash = calculateHash(profileJson)
        profilesHashes[filename] = itemHash
        print('Hash for %s - %s' % (filename, itemHash))

    print('Writing profilesHashes.json')
    profilesHashesJson = convertDataToJson(profilesHashes)
    saveToFile(profilesHashesJson, outputPath, 'profilesHashes.json')

def generateParametersJson(outputPath):
    print('-- Converting parameters.plist to json --')

    parametersPath = os.path.join(currentdir, PARAMETERS_PLIST_PATH)
    parametersPlist = readPlist(parametersPath)

    print('Writing parameters.json')
    parametersJson = convertDataToJson(parametersPlist)
    saveToFile(parametersJson, outputPath, 'parameters.json')

    print('Calculating parameters hash')
    parametersHash = calculateHash(parametersJson)
    print('Hash for parameters - %s' % parametersHash)

    print('Writing parametersHash.txt')
    saveToFile(parametersHash, outputPath, 'parametersHash.txt')

def generateDownloadsJson(outputPath, platform):
    print('-- Minifying downloads json  --')

    downloadsPath = os.path.join(currentdir, DOWNLOADS_PATH)

    print('Reading the downloads json')
    downloadsPlist = readPlist(downloadsPath)
    downloadsJson = convertDataToJson(downloadsPlist)

    print('Writing minified downloads.json')
    saveToFile(downloadsJson, outputPath, 'downloads.json')
 
    print('Calculating downloads hash')
    downloadsHash = calculateHash(downloadsJson)
    print('Hash for downloads - %s' % downloadsHash)

    print('Writing downloadsHash.txt')
    saveToFile(downloadsHash, outputPath, 'downloadsHash.txt')

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Package game data', add_help=False)
    parser.add_argument('platform', metavar='P', help='ios or android')
 
    args = parser.parse_args()

    currentdir = os.path.dirname(os.path.realpath(__file__))
    outputPath = os.path.join(currentdir, PROFILES_OUTPUT_PATH)
    
    if not os.path.exists(outputPath):
        os.makedirs(outputPath)

    generateProfilesJson(outputPath)
    generateParametersJson(outputPath)
    generateDownloadsJson(outputPath, args.platform)
